import numpy as np
import posixshmem


def parse_shmem_handle(handle_string):
    buffer_name, dtype, shape, index = handle_string.split("$")
    dtype = getattr(np, dtype)
    shape = tuple(int(n) for n in shape.split(","))
    index = int(index)
    return buffer_name, dtype, shape, index


def open_shmem_from_handle(handle_string, rw=False):
    buffer_name, dtype, shape, _ = parse_shmem_handle(handle_string)
    buffer_mem_size = np.dtype(dtype).itemsize * np.product(shape)

    buffer_mem = posixshmem.SharedMemory(name=buffer_name, size=buffer_mem_size, rw=rw)

    array = buffer_mem.ndarray(
        shape=shape,
        dtype=dtype,
    )

    # returning both; in case of rw (why), user should close memory allocation
    return buffer_mem, array


def handle_template_from_shmem_array(mem, ary):
    shape_str = ",".join(str(n) for n in ary.shape)
    return f"{mem.name}${ary.dtype}${shape_str}${{index}}"
